import math

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.widgets import Slider

from ...plots import generate_mask_colors, plot_contour
from ...utils import NumLst, PathTypeLst, StrLst, mask_path_to_data_ax, path_in


def masks_path(
    paths: PathTypeLst,
    rows: int or None = None,
    columns: int or None = None,
    titles: StrLst = [],
    mask_colors=None,
    origin="upper",
):
    if len(paths) == 0:
        raise ValueError("paths must be a list of paths")

    paths = [path_in(path) for path in paths]

    masks_data = [mask_path_to_data_ax(path) for path in paths]

    masks_array(
        masks_data=masks_data,
        rows=rows,
        columns=columns,
        titles=titles,
        mask_colors=mask_colors,
        origin=origin,
    )


def masks_array(
    masks_data: NumLst,
    rows=None,
    columns=None,
    titles=[],
    mask_colors=[],
    origin="upper",
):
    print("Loading images...")

    if len(masks_data) == 0:
        raise ValueError("images_data must be a list of arrays")

    for mask_data in masks_data:
        if mask_data.ndim != 3:
            raise ValueError("images_data must be a list of 3D arrays")

    num_masks = len(masks_data)  # Number of images

    if rows is None and columns is None:
        rows = math.ceil(math.sqrt(num_masks))  # Number of rows in the grid
        columns = math.ceil(num_masks / rows)  # Number of columns in the grid
    elif rows is None:
        rows = math.ceil(num_masks / columns)
    elif columns is None:
        columns = math.ceil(num_masks / rows)

    if num_masks > rows * columns:
        raise ValueError("rows * columns must be greater than or equal to num_images")

    depth = [mask_data.shape[2] for mask_data in masks_data]

    init_slice, last_slice = int(np.mean(depth) // 2), np.max(depth) - 2

    mask_colors = generate_mask_colors(num_masks, mask_colors)

    _, axs = plt.subplots(rows, columns)
    if num_masks == 1:
        axs = np.array([axs])

    plt.subplots_adjust(bottom=0.25)

    for i, ax in enumerate(axs.flat):
        if i < num_masks:
            title = titles[i] if titles else f"Image {i}"

            plot_contour(
                ax,
                masks_data[i][:, :, init_slice],
                color=mask_colors[i],
                title=title,
                origin=origin,
                levels=[0.5],
            )
            # ax.axis("off")
            ax.set_xlabel(f"Slice Number: {init_slice}")

        else:
            ax.axis("off")

    slider_ax = plt.axes([0.2, 0.1, 0.6, 0.03])

    slider = Slider(
        slider_ax,
        "Slice",
        0,
        last_slice,
        valinit=init_slice,
        valstep=1,
    )

    def update(val):
        slice_num = int(slider.val)

        for i, ax in enumerate(axs.flat):
            ax.clear()

            if i < num_masks:
                title = titles[i] if titles else f"Image {i}"

                if masks_data[i].shape[2] <= slice_num:
                    slice_num = masks_data[i].shape[2] - 1

                plot_contour(
                    ax,
                    masks_data[i][:, :, slice_num],
                    color=mask_colors[i],
                    title=title,
                    origin=origin,
                    levels=[0.5],
                )
                ax.set_xlabel(f"Slice Number: {slice_num}")
                # ax.axis("off")
            else:
                ax.axis("off")
                pass
        # ax.set_title(title)

    slider.on_changed(update)

    plt.tight_layout()
    plt.show()



"""
INPUT: 3D image, 3D mask
Finds slices with the most values 

OUTPU: 2D images
"""

import math

import matplotlib.pyplot as plt
import numpy as np

from ...plots import generate_mask_colors, plot_contour, plot_image, save_image
from ...utils import (
    PathType,
    image_path_to_data_ax,
    mask_path_to_data_ax,
    path_in,
    save_path_file,
    significant_slice_idx_data,
)


def image_mask_annotated_path(
    image_path: PathType,
    mask_path: PathType,
    cmap="gray",
    save_path=None,
    limit: int = 10,
):
    image_path = path_in(image_path)
    mask_path = path_in(mask_path)

    image_data = image_path_to_data_ax(image_path)
    mask_data = mask_path_to_data_ax(mask_path)

    image_mask_annotated_array(
        image_data=image_data,
        mask_data=mask_data,
        cmap=cmap,
        save_path=save_path,
        limit=limit,
    )


def image_mask_annotated_array(
    image_data,
    mask_data,
    cmap="gray",
    save_path=None,
    limit: int = 10,
):
    print("Loading images...")

    most_value_nonzero_slices, num_nonzero_slices = significant_slice_idx_data(
        mask_data
    )
    num_masks = min(num_nonzero_slices, limit)

    rows = math.ceil(math.sqrt(num_masks))  # Number of rows in the grid
    columns = math.ceil(num_masks / rows)  # Number of columns in the grid

    mask_colors = generate_mask_colors(num_masks)

    _, axs = plt.subplots(rows, columns)

    if num_masks == 1:
        axs = np.array([axs])

    for i, ax in enumerate(axs.flat):
        if i < num_masks:
            # title = titles[i] if titles else f"Image {i}"
            plot_image(ax, image_data[:, :, most_value_nonzero_slices[i]], cmap=cmap)
            plot_contour(
                ax,
                mask_data[:, :, most_value_nonzero_slices[i]],
                color=mask_colors[i],
                levels=[0.5],
            )
            ax.set_title(f"Slice {most_value_nonzero_slices[i]}")
            # ax.axis("off")
        else:
            ax.axis("off")

    plt.tight_layout()

    if save_path:
        save_path = save_path_file(save_path, suffix=".png")
        save_image(plt, save_path)
    else:
        plt.show()

    plt.close()



from .images import images_array, images_path
from .masks import masks_array, masks_path
from .image_masks import image_masks_array, image_masks_path
from .image_mask_annotated import image_mask_annotated_array, image_mask_annotated_path



import math

import matplotlib.pyplot as plt
import numpy as np

from ...plots import generate_mask_colors, plot_contour, save_image
from ...utils import (
    NumLst,
    PathType,
    PathTypeLst,
    StrLst,
    mask_path_to_data_ax,
    path_in,
    save_path_file,
)


def masks_path(
    paths: PathTypeLst,
    rows: int or None = None,
    columns: int or None = None,
    titles: StrLst = [],
    mask_colors=None,
    origin: str or None = "upper",
    save_path: PathType or None = None,
):
    if len(paths) == 0:
        raise ValueError("paths must be a list of paths")

    paths = [path_in(path) for path in paths]
    masks_data = [mask_path_to_data_ax(path) for path in paths]  # just for nifti files

    masks_array(
        masks_data=masks_data,
        rows=rows,
        columns=columns,
        titles=titles,
        origin=origin,
        mask_colors=mask_colors,
        save_path=save_path,
    )


def masks_array(
    masks_data: NumLst,
    rows=None,
    columns=None,
    titles=[],
    mask_colors=None,
    origin="upper",
    save_path=None,
):
    print("Loading images...")

    if len(masks_data) == 0:
        raise ValueError("masks_data must be a list of arrays")

    for mask_data in masks_data:
        if mask_data.ndim != 2:
            raise ValueError("Masks must be 2D")

    num_masks = len(masks_data)  # Number of images

    if rows is None and columns is None:
        rows = math.ceil(math.sqrt(num_masks))  # Number of rows in the grid
        columns = math.ceil(num_masks / rows)  # Number of columns in the grid
    elif rows is None:
        rows = math.ceil(num_masks / columns)
    elif columns is None:
        columns = math.ceil(num_masks / rows)

    if num_masks > rows * columns:
        raise ValueError("rows * columns must be greater than or equal to num_images")

    mask_colors = generate_mask_colors(num_masks, mask_colors)

    _, axs = plt.subplots(rows, columns)
    if num_masks == 1:
        axs = np.array([axs])

    for i, ax in enumerate(axs.flat):
        if i < num_masks:
            title = titles[i] if titles else f"Mask {i}"

            plot_contour(
                ax,
                masks_data[i],
                color=mask_colors[i],
                title=title,
                origin=origin,
                levels=[0.5],
            )
            # ax.axis("off")
        else:
            ax.axis("off")

    plt.tight_layout()

    if save_path:
        save_path = save_path_file(save_path, suffix=".png")
        save_image(plt, save_path)
    else:
        plt.show()

    plt.close()



import math

import matplotlib.pyplot as plt
import numpy as np

from ...plots import plot_image, save_image
from ...utils import (
    NumLst,
    PathType,
    PathTypeLst,
    StrLst,
    image_path_to_data_ax,
    path_in,
    save_path_file,
)


def images_path(
    paths: PathTypeLst,
    rows: int or None = None,
    columns: int or None = None,
    titles: StrLst = [],
    cmap: str or None = "gray",
    origin: str or None = "upper",
    save_path: PathType or None = None,
):
    if len(paths) == 0:
        raise ValueError("paths must be a list of paths")

    paths = [path_in(path) for path in paths]

    images_data = [image_path_to_data_ax(path) for path in paths]

    images_array(
        images_data=images_data,
        rows=rows,
        columns=columns,
        titles=titles,
        cmap=cmap,
        origin=origin,
        save_path=save_path,
    )


def images_array(
    images_data: NumLst,
    rows=None,
    columns=None,
    titles=[],
    cmap="gray",
    origin="upper",
    save_path=None,
):
    print("Loading images...")

    if len(images_data) == 0:
        raise ValueError("images_data must be a list of arrays")

    for image_data in images_data:
        if image_data.ndim != 2:
            raise ValueError("Images must be 2D")

    num_images = len(images_data)  # Number of images

    if rows is None and columns is None:
        rows = math.ceil(math.sqrt(num_images))  # Number of rows in the grid
        columns = math.ceil(num_images / rows)  # Number of columns in the grid
    elif rows is None:
        rows = math.ceil(num_images / columns)
    elif columns is None:
        columns = math.ceil(num_images / rows)

    if num_images > rows * columns:
        raise ValueError("rows * columns must be greater than or equal to num_images")

    _, axs = plt.subplots(rows, columns)
    if num_images == 1:
        axs = np.array([axs])

    for i, ax in enumerate(axs.flat):
        if i < num_images:
            title = titles[i] if titles else f"Image {i}"
            plot_image(ax, images_data[i], cmap=cmap, title=title, origin=origin)
            ax.axis("off")

        else:
            ax.axis("off")

    plt.tight_layout()

    if save_path:
        save_path = save_path_file(save_path, suffix=".png")
        save_image(plt, save_path)
    else:
        plt.show()

    plt.close()



# from .utils.helper_mask import *
# from .plots.helper_plot import *

# # from .preprocess.mask import generate_mask_schema

# # from .plots.layered_plot import *
# # from .plots.layered_plot2D import *
# # from .plots.plot_image import *
# # from .plots.plot3d.plot_images import *
# from .preprocess.mask import generate_mask_schema
# from .preprocess.match_image_mask import match_image_mask

from .utils import (
    image_path_to_data_ax,
    image_path_to_data_sag,
    mask_path_to_data_ax,
    mask_path_to_data_sag,
    save_path_dir,
    save_path_file,
    path_in,
    im2arr,
    read_image_mask,
)


from .utils.array_profile import profile
from .utils.array_threshold import compute_thresholds

# from .utils import significant_slices

### PLOT 2D ###
# from .plots.plot2d.images import (
#     images_array as plot2d_images_array,
#     images_path as plot2d_images_path,
# )

# from .plots.plot2d.masks import (
#     masks_array as plot2d_masks_array,
#     masks_path as plot2d_masks_path,
# )

# from .plots.plot2d.image_masks import (
#     image_masks_path as plot2d_image_masks_path,
#     image_masks_array as plot2d_image_masks_array,
# )


# from .plots.plot2d.image_mask_annotated import (
#     image_mask_annotated_array as plot2d_image_mask_annotated_array,
#     image_mask_annotated_path as plot2d_image_mask_annotated_path,
# )

### PLOT 3D ###


# from .plots.plot3d import layered_slider_array, layered_slider_path

# from .plots.plot3d.images import (
#     images_array as plot3d_images_array,
#     images_path as plot3d_images_path,
# )

# from .plots.plot3d.masks import (
#     masks_array as plot3d_masks_array,
#     masks_path as plot3d_masks_path,
# )

# from .plots.plot3d.image_masks import (
#     image_masks_path as plot3d_image_masks_path,
#     image_masks_array as plot3d_image_masks_array,
# )

# from .plots.plot3d.layered_plot import (
#     layered_plot_data3D as plot3d_layered_data,
#     layered_plot_path3D as plot3d_layered_path,
# )

# from .plots.gif import gif, gif_slices, gif_non, gif_reference


from .multimodal import save_dicom_metadata, filter_dicom

from .preprocess import *

from .feats import *

# from .nifti.helper import nii3d_to_annotated2d, nii_mask3d_to_2d

# def version():
#     with open("VERSION", "r") as f:
#         version = f.read().strip()
#     return version

# __version__ = version()

__version__ = "1.1.3"



from filecmp import cmp

import matplotlib.pyplot as plt

from ...plots import (
    assert_shape,
    generate_mask_colors,
    plot_contour,
    plot_image,
    save_image,
)
from ...utils import (
    image_path_to_data_ax,
    mask_path_to_data_ax,
    path_in,
    save_path_file,
)


def image_masks_path(
    image_path,
    masks_path,
    mask_colors=None,
    titles=[],
    title_image="Image",
    cmap="gray",
    origin="upper",
    save_path=None,
):
    if len(masks_path) == 0:
        raise ValueError("masks_path must be a list of paths")

    image_path = path_in(image_path)
    image_data = image_path_to_data_ax(image_path)

    masks_path = [path_in(path) for path in masks_path]
    masks_data = [mask_path_to_data_ax(path) for path in masks_path]

    image_masks_array(
        image_data=image_data,
        masks_data=masks_data,
        mask_colors=mask_colors,
        titles=titles,
        title_image=title_image,
        cmap=cmap,
        origin=origin,
        save_path=save_path,
    )


def image_masks_array(
    image_data,
    masks_data,
    mask_colors=None,
    titles=[],
    title_image="Image",
    cmap="gray",
    origin="upper",
    save_path=None,
):
    if len(masks_data) == 0:
        raise ValueError("masks_data must be a list of arrays")

    print("Loading images...")

    if image_data.ndim != 2:
        raise ValueError("Image must be 2D")

    for mask_data in masks_data:
        if mask_data.ndim != 2:
            raise ValueError("Masks must be 2D")

    num_masks = len(masks_data)

    # assert_shape(image_data + masks_data)

    mask_colors = generate_mask_colors(num_masks, mask_colors)

    _, ax = plt.subplots()
    plt.subplots_adjust(bottom=0.25)

    plot_image(ax, image_data, cmap=cmap, origin=origin, title=title_image)

    for i in range(num_masks):
        title = titles[i] if titles else f"Mask {i}"
        plot_contour(
            ax,
            masks_data[i],
            color=mask_colors[i],
            title=title,
            origin=origin,
            levels=[0.5],
        )

    # ax.set_xlabel(f"Slice Number: {slice}")
    # ax.set_title(title)
    plt.tight_layout()

    if save_path:
        save_path = save_path_file(save_path, suffix=".png")
        save_image(plt, save_path)
    else:
        plt.show()

    plt.close()
